﻿#include "StdAfx.h"
#include "SettingPersonal.h"
#include "UIMenu.h"
#include "main_frame.hpp"
#include "NeMessengerDlg.h"
#include "FavoriteController.h"
#include "SettingFrame.hpp"

#define MAX_AWAY_COUNT 4
const int c_nAwayTime[MAX_AWAY_COUNT] = { 5, 10, 30,60};

static bool OnTextEvent(void* event) 
{
	TEventUI *pEvent = (TEventUI*)event;
	static CString strOldImage = _T("");
	if( ((TEventUI*)event)->Type == UIEVENT_BUTTONDOWN ) 
	{
		CControlUI* pCtrl= ((TEventUI*)event)->pSender;
		CSettingPersonal* pThis = (CSettingPersonal*)pCtrl->GetTag();
		pThis->ProcLabelEvent(pCtrl);

	}
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSEENTER)
	{
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
	}
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSELEAVE)
	{
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
	}
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSEHOVER)
	{
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
	}
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSEMOVE)
	{
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
	}

	return true;
}

CSettingPersonal::CSettingPersonal(CSettingFrame *pSettingFrame, CPaintManagerUI& paint_manager)
: m_paintManager(paint_manager)
{
	m_pSettingFrame = pSettingFrame;
	m_bTimeCheck = false;
	m_nAwayTimeSpin = GET_CONFIG()->GetAutoAwayTime();
	m_nGroupType = 0;
	m_pListFavorite = NULL;
	m_pListExport = NULL;
	m_pListImport = NULL;

	m_pFavoriteRootNode = new Node;
	m_pFavoriteRootNode->data().level_ = -1;
	m_pFavoriteRootNode->data().child_visible_ = true;
	m_pFavoriteRootNode->data().has_child_ = true;
	m_pFavoriteRootNode->data().list_elment_ = NULL;

	m_pExportRootNode = new Node;
	m_pExportRootNode->data().level_ = -1;
	m_pExportRootNode->data().child_visible_ = true;
	m_pExportRootNode->data().has_child_ = true;
	m_pExportRootNode->data().list_elment_ = NULL;

	m_pImportRootNode = new Node;
	m_pImportRootNode->data().level_ = -1;
	m_pImportRootNode->data().child_visible_ = true;
	m_pImportRootNode->data().has_child_ = true;
	m_pImportRootNode->data().list_elment_ = NULL;
	
	CreateControl();
}

CSettingPersonal::~CSettingPersonal()
{
	if (m_pFavoriteRootNode != NULL)
	{
		delete m_pFavoriteRootNode;
		m_pFavoriteRootNode = NULL;
	}
	if (m_pExportRootNode != NULL)
	{
		delete m_pExportRootNode;
		m_pExportRootNode = NULL;
	}
	if (m_pImportRootNode != NULL)
	{
		delete m_pImportRootNode;
		m_pImportRootNode = NULL;
	}
}

void CSettingPersonal::CreateControl()
{
	CPaintManagerUI paint_manager;
	CDialogBuilder builder;
	CContainerUI* pContainerUI = static_cast<CContainerUI*>(builder.Create(_T("setting_personal.xml"),  (UINT)0, (UINT)0, &m_paintManager));
	if( pContainerUI != NULL ) 
	{
		this->Add(pContainerUI);
	}
	else 
	{
		this->RemoveAll();
		return;
	}
}

void CSettingPersonal::InitControl()
{
	m_pListFavorite = static_cast<CListUI*>(m_pManager->FindControl(kSettingPersonalListFavorite));
	m_pListExport = static_cast<CListUI*>(m_pManager->FindControl(kSettingPersonalListExport));
	m_pListImport = static_cast<CListUI*>(m_pManager->FindControl(kSettingPersonalListImport));
	ASSERT(m_pListFavorite != NULL && m_pListExport != NULL && m_pListImport != NULL);
	InitData();
	ShowTimeCheckStatus();
	EnableTimeCombo();
}

void CSettingPersonal::InitData()
{
	m_bTimeCheck = GET_CONFIG()->GetAutoAwayState() == 1 ? true : false;

	CLabelUI* pLabUserName = static_cast<CLabelUI*>(m_pManager->FindControl(_T("personal_username")));
	if (pLabUserName != NULL)
	{
		pLabUserName->SetText(LoadStringFromResource(IDS_STRING424));
	}
	CEditUI* pEditUserName = static_cast<CEditUI*>(m_pManager->FindControl(_T("personal_edit_name")));
	if (pEditUserName != NULL)
	{
		CString strText = GET_CONFIG()->GetNickName();
		pEditUserName->SetText(strText);
		SetNameLength(strText.GetLength());
	}
	CLabelUI* pStatus = static_cast<CLabelUI*>(m_pManager->FindControl(_T("personal_status")));
	if (pStatus != NULL)
	{
		pStatus->SetText(LoadStringFromResource(IDS_STRING425));
	}

	CComboUI* pTimeCombo = static_cast<CComboUI*>(m_pManager->FindControl(kSettingPersonalTimeCombo));
	if (pTimeCombo != NULL)
	{
		BOOL bFind = FALSE;
		for (int i=0; i<MAX_AWAY_COUNT; ++i)
		{
			if (c_nAwayTime[i] == GET_CONFIG()->GetAutoAwayTime())
			{
				pTimeCombo->SetInternVisible();
				pTimeCombo->SelectItem(i);
				bFind = TRUE;
				break;
			}
		}
		if (!bFind)
		{
			pTimeCombo->SetInternVisible();
			pTimeCombo->SelectItem(0);
		}
	}
	COptionUI* pExportGroup = static_cast<COptionUI*>(m_pManager->FindControl(kSettingPersonalExportGroup));
	if (pExportGroup != NULL)
	{
		pExportGroup->SetText(LoadStringFromResource(IDS_STRING_EXPORT_GROUPS));
	}
	COptionUI* pImportGroup = static_cast<COptionUI*>(m_pManager->FindControl(kSettingPersonalImportGroup));
	if (pImportGroup != NULL)
	{
		pImportGroup->SetText(LoadStringFromResource(IDS_STRING_IMPORT_GROUPS));
	}
	CLabelUI* pLabFavorite = static_cast<CLabelUI*>(m_pManager->FindControl(kSettingPersonalLabFavorite));
	if (pLabFavorite != NULL)
	{
		pLabFavorite->SetText(LoadStringFromResource(IDS_STRING_FAVORITE_LIST));
	}
	CLabelUI* pLabExport = static_cast<CLabelUI*>(m_pManager->FindControl(kSettingPersonalLabExport));
	if (pLabExport != NULL)
	{
		pLabExport->SetText(LoadStringFromResource(IDS_STRING_EXPORT_LIST));
	}
	CButtonUI* pBtnFile = static_cast<CButtonUI*>(m_pManager->FindControl(kSettingPersonalFile));
	if (pBtnFile != NULL)
	{
		pBtnFile->SetText(LoadStringFromResource(IDS_STRING_FILE_SAVE));
	}

	size_t nGroupCount = g_pMessengerActor->GetBuddyGroupCount();
	for (size_t i = 0; i < nGroupCount; i++)
	{
		GroupItemInfo item;
		const IGTBuddyGroup *pGroup = g_pMessengerActor->GetBuddyGroupInfoFromIndex(i);
		item.strGroupName = pGroup->GetName();
		item.nGroupID = pGroup->GetID();
		
		AddFavoriteNode(item, NULL);
	}
}

Node* CSettingPersonal::AddFavoriteNode(const GroupItemInfo &item, Node* parent, BOOL bInsertFirst)
{
	if (!parent)
		parent = m_pFavoriteRootNode;

    CListContainerElementUI* pListElement = NULL;
    if( !m_dlgBuilder.GetMarkup()->IsValid() ) 
	{
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("PersonalGroupListItem.xml"), (UINT)0, NULL, m_pManager));
    }
    else 
	{
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, m_pManager));
    }
    if (pListElement == NULL)
        return NULL;

	Node* node = new Node;

	node->data().level_ = parent->data().level_ + 1;
	node->data().has_child_ = false;
	node->data().folder_ = false;
	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;
	node->data().text_ = item.strGroupName;
	node->data().value.Format(_T("%d"), item.nGroupID);
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != m_pFavoriteRootNode && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CLabelUI* pGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pListElement, kSettingPersonalGroupName));
	if (pGroupName != NULL)
	{
		pGroupName->SetText(item.strGroupName);
		pGroupName->SetToolTip(item.strGroupName);
	}
	int index = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
	}
	else 
	{
		if (parent == m_pFavoriteRootNode)
			index = 0;
		else
			index = parent->data().list_elment_->GetIndex() + 1;
	}
	
	CDuiRect rcPadding(0,0,0,0);
	if (bInsertFirst)
	{
		index = 0;
		rcPadding.top = 7;
	}
	if (m_pFavoriteRootNode->num_children() < 1 )
	{
		rcPadding.top = 7;
	}
	pListElement->SetPadding(rcPadding);
	pListElement->SetSelectPadding(CRect(3,0,1,3));
	pListElement->SetFixedHeight(20);
	pListElement->SetTag((UINT_PTR)node);
	if (!m_pListFavorite->AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}
	if (node != NULL)
	{
		parent->add_child(node);
	}
	
	return node;
}

Node* CSettingPersonal::AddImportNode(const GroupItemInfo &item, Node* parent)
{
	if (!parent)
		parent = m_pImportRootNode;

	CListContainerElementUI* pListElement = NULL;
	if( !m_dlgBuilder.GetMarkup()->IsValid() ) 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("PersonalGroupListItem.xml"), (UINT)0, NULL, m_pManager));
	}
	else 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, m_pManager));
	}
	if (pListElement == NULL)
		return NULL;

	Node* node = new Node;

	node->data().level_ = parent->data().level_ + 1;
	node->data().has_child_ = false;
	node->data().folder_ = false;
	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;
	node->data().text_ = item.strGroupName;
	node->data().value.Format(_T("%d"), item.nGroupID);
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != m_pImportRootNode && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CLabelUI* pGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pListElement, kSettingPersonalGroupName));
	if (pGroupName != NULL)
	{
		pGroupName->SetText(item.strGroupName);
		pGroupName->SetToolTip(item.strGroupName);
	}
	CDuiRect rcPadding(0,0,0,0);
	if (m_pImportRootNode->num_children() < 1 )
	{
		rcPadding.top = 7;
	}
	pListElement->SetPadding(rcPadding);
	pListElement->SetSelectPadding(CRect(3,0,1,3));
	pListElement->SetFixedHeight(20);
	pListElement->SetTag((UINT_PTR)node);
	int index = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
	}
	else 
	{
		if (parent == m_pImportRootNode)
			index = 0;
		else
			index = parent->data().list_elment_->GetIndex() + 1;
	}
	if (!m_pListImport->AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}
	if (node != NULL)
	{
		parent->add_child(node);
	}

	return node;
}

Node* CSettingPersonal::AddExportNode(const GroupItemInfo &item, Node* parent)
{
	if (!parent)
		parent = m_pExportRootNode;

	CListContainerElementUI* pListElement = NULL;
	if( !m_dlgBuilder.GetMarkup()->IsValid() ) 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("PersonalGroupListItem.xml"), (UINT)0, NULL, m_pManager));
	}
	else 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, m_pManager));
	}
	if (pListElement == NULL)
		return NULL;

	Node* node = new Node;

	node->data().level_ = parent->data().level_ + 1;
	node->data().has_child_ = false;
	node->data().folder_ = false;
	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;
	node->data().text_ = item.strGroupName;
	node->data().value.Format(_T("%d"), item.nGroupID);
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != m_pExportRootNode && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CLabelUI* pGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pListElement, kSettingPersonalGroupName));
	if (pGroupName != NULL)
	{
		pGroupName->SetText(item.strGroupName);
		pGroupName->SetToolTip(item.strGroupName);
	}
	CDuiRect rcPadding(0,0,0,0);
	if (m_pExportRootNode->num_children() < 1 )
	{
		rcPadding.top = 7;
	}
	pListElement->SetPadding(rcPadding);
	pListElement->SetSelectPadding(CRect(3,0,1,3));
	pListElement->SetFixedHeight(20);
	pListElement->SetTag((UINT_PTR)node);
	int index = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
	}
	else 
	{
		if (parent == m_pExportRootNode)
			index = 0;
		else
			index = parent->data().list_elment_->GetIndex() + 1;
	}
	if (!m_pListExport->AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}
	if (node != NULL)
	{
		parent->add_child(node);
	}

	return node;
}

bool CSettingPersonal::RemoveFavoriteNode(Node* node)
{
	if (!node || node == m_pFavoriteRootNode) return false;

	for (int i = 0; i < node->num_children(); ++i)
	{
		Node* child = node->child(i);
		RemoveFavoriteNode(child);
	}
	
	m_pListFavorite->Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;

	return true;
}

bool CSettingPersonal::RemoveExportNode(Node* node)
{
	if (!node || node == m_pExportRootNode) return false;

	for (int i = 0; i < node->num_children(); ++i)
	{
		Node* child = node->child(i);
		RemoveExportNode(child);
	}

	m_pListExport->Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;

	return true;
}

void CSettingPersonal::RemoveAllMenberList(CListUI* pList)
{
	pList->RemoveAll();
	for (int i = 0; i < m_pFavoriteRootNode->num_children(); ++i)
	{
		Node* child = m_pFavoriteRootNode->child(i);
		RemoveFavoriteNode(child);
	}
	delete m_pFavoriteRootNode;
	m_pFavoriteRootNode = new Node;
	m_pFavoriteRootNode->data().level_ = -1;
	m_pFavoriteRootNode->data().child_visible_ = true;
	m_pFavoriteRootNode->data().has_child_ = true;
	m_pFavoriteRootNode->data().list_elment_ = NULL;
}

list<Node*> CSettingPersonal::GetSelectedNodeList(CListUI* pList, BOOL bEnable)
{
	list<Node*> listSelectedNode;
	list<int> listSelectedIndexs = pList->GetSelectedItems();
	for(list<int>::iterator it = listSelectedIndexs.begin(); it != listSelectedIndexs.end(); ++it)
	{
		CControlUI* pControl = pList->GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL && pControl->IsEnabled())
			{
				listSelectedNode.push_back(node);
			}
			if (!bEnable)
			{
				CLabelUI* pLabGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pControl, kSettingPersonalGroupName));
				pLabGroupName->SetEnabled(false);
				pControl->SetEnabled(false);
			}
			
		}
	}
	return listSelectedNode;
}

void CSettingPersonal::NotifyCtrlClick(TNotifyUI& msg)
{
	if (_tcsicmp(msg.pSender->GetName(), kSettingPersonalTimeCheck) == 0)
	{
		m_bTimeCheck = !m_bTimeCheck;
		ShowTimeCheckStatus();
		EnableTimeCombo();
	}
	else if (_tcsicmp(msg.pSender->GetName(), _T("personal_btn_top")) == 0)
	{
		if (m_nGroupType == PERSONAL_GROUP_EXPORT)
		{
			list<Node*> listSelectedNode = GetSelectedNodeList(m_pListFavorite);
			for (list<Node*>::iterator it = listSelectedNode.begin(); it != listSelectedNode.end(); ++it)
			{
				GroupItemInfo item;
				item.nGroupID = _ttoi((*it)->data().value.GetData());
				item.strGroupName = (*it)->data().text_;
				vector<CString>::iterator itFind = find(m_vecExportList.begin(), m_vecExportList.end(), item.strGroupName);
				if (itFind == m_vecExportList.end())
				{
					AddExportNode(item, NULL);
					m_vecExportList.push_back(item.strGroupName);
				}
			}
		}
		else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
		{
			list<Node*> listSelectedNode = GetSelectedNodeList(m_pListFavorite);
			for (list<Node*>::iterator it = listSelectedNode.begin(); it != listSelectedNode.end(); ++it)
			{
				CString strGroup = (*it)->data().text_.GetData();
				vector<GroupItemInfo>::iterator itFind = m_vecAddToFavorite.begin();
				for (; itFind!=m_vecAddToFavorite.end(); ++itFind)
				{
					if ((*itFind).strGroupName == strGroup)
					{
						m_vecAddToFavorite.erase(itFind);
						RemoveFavoriteNode((*it));
						break;
					}
				}
			}
			CControlUI* pCtrlFirstItem = m_pListFavorite->GetItemAt(0);
			if (pCtrlFirstItem != NULL)
			{
				CDuiRect rcPadding(0,7,0,0);
				pCtrlFirstItem->SetPadding(rcPadding);
			}
		}
		
	}
	else if (_tcsicmp(msg.pSender->GetName(), _T("personal_btn_down")) == 0)
	{
		if (m_nGroupType == PERSONAL_GROUP_EXPORT)
		{
			list<Node*> listSelectedNode = GetSelectedNodeList(m_pListExport);
			for (list<Node*>::iterator it = listSelectedNode.begin(); it != listSelectedNode.end(); ++it)
			{
				CString strGroupName = (*it)->data().text_.GetData();
				vector<CString>::iterator itFind = find(m_vecExportList.begin(), m_vecExportList.end(), strGroupName);
				if (itFind != m_vecExportList.end())
				{
					m_vecExportList.erase(itFind);
					RemoveExportNode(*it);
				}
			}
			CControlUI* pCtrlFirstItem = m_pListExport->GetItemAt(0);
			if (pCtrlFirstItem != NULL)
			{
				CDuiRect rcPadding(0,7,0,0);
				pCtrlFirstItem->SetPadding(rcPadding);
			}
		}
		else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
		{
			FavoriteController* pFavoriteCtrl = GET_MESSENGER_DLG()->GetMainFrame()->GetFavoriteController();
			list<Node*> listSelectedNode = GetSelectedNodeList(m_pListImport);
			for (list<Node*>::iterator it = listSelectedNode.begin(); it != listSelectedNode.end(); ++it)
			{
				GroupItemInfo item;
				item.nGroupID = _ttoi((*it)->data().value.GetData());
				item.strGroupName = GetNewGroupName((*it)->data().text_.GetData());
				m_vecAddToFavorite.push_back(item);
				AddFavoriteNode(item, NULL, TRUE);
				CControlUI* pCtrlFirstItem = m_pListFavorite->GetItemAt(1);
				if (pCtrlFirstItem != NULL)
				{
					CDuiRect rcPadding(0,1,0,0);
					pCtrlFirstItem->SetPadding(rcPadding);
				}
			}
		}
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingPersonalFile) == 0)
	{
		if (m_nGroupType == PERSONAL_GROUP_EXPORT)
		{
			SaveFavoriteFile();
		}
		else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
		{
			ImportFavoriteFile();
		}
	}
}

void CSettingPersonal::OnNotifyItemDoubleClick(TNotifyUI& msg)
{
	CPoint pt(msg.ptMouse);
	if (m_nGroupType == PERSONAL_GROUP_EXPORT)
	{
		Node* pNodeItem = (Node*)msg.pSender->GetTag();
		if (pNodeItem != NULL)
		{
			CRect rcFavorit = m_pListFavorite->GetPos();
			CRect rcExport = m_pListExport->GetPos();
			if (rcFavorit.PtInRect(pt))
			{
				GroupItemInfo item;
				item.nGroupID = _ttoi(pNodeItem->data().value.GetData());
				item.strGroupName = pNodeItem->data().text_;
				vector<CString>::iterator itFind = find(m_vecExportList.begin(), m_vecExportList.end(), item.strGroupName);
				if (itFind == m_vecExportList.end())
				{
					AddExportNode(item, NULL);
					m_vecExportList.push_back(item.strGroupName);
				}
			}
			else if (rcExport.PtInRect(pt))
			{
				CString strGroupName = pNodeItem->data().text_.GetData();
				vector<CString>::iterator itFind = find(m_vecExportList.begin(), m_vecExportList.end(), strGroupName);
				if (itFind != m_vecExportList.end())
				{
					m_vecExportList.erase(itFind);
					RemoveExportNode(pNodeItem);
				}
				CControlUI* pCtrlFirstItem = m_pListExport->GetItemAt(0);
				if (pCtrlFirstItem != NULL)
				{
					CDuiRect rcPadding(0,7,0,0);
					pCtrlFirstItem->SetPadding(rcPadding);
				}
			}
		}
	}
	else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
	{
		Node* pNodeItem = (Node*)msg.pSender->GetTag();
		if (pNodeItem != NULL)
		{
			CRect rcFavorit = m_pListFavorite->GetPos();
			CRect rcImport = m_pListImport->GetPos();
			if (rcFavorit.PtInRect(pt))
			{
				CString strGroup = pNodeItem->data().text_.GetData();
				vector<GroupItemInfo>::iterator itFind = m_vecAddToFavorite.begin();
				for (; itFind!=m_vecAddToFavorite.end(); ++itFind)
				{
					if ((*itFind).strGroupName == strGroup)
					{
						m_vecAddToFavorite.erase(itFind);
						RemoveFavoriteNode(pNodeItem);
						break;
					}
				}

				CControlUI* pCtrlFirstItem = m_pListFavorite->GetItemAt(0);
				if (pCtrlFirstItem != NULL)
				{
					CDuiRect rcPadding(0,7,0,0);
					pCtrlFirstItem->SetPadding(rcPadding);
				}
			}
			else if (rcImport.PtInRect(pt))
			{
				GroupItemInfo item;
				item.nGroupID = _ttoi(pNodeItem->data().value.GetData());
				item.strGroupName = GetNewGroupName(pNodeItem->data().text_.GetData());
				m_vecAddToFavorite.push_back(item);
				AddFavoriteNode(item, NULL, TRUE);
				CControlUI* pCtrlFirstItem = m_pListFavorite->GetItemAt(1);
				if (pCtrlFirstItem != NULL)
				{
					CDuiRect rcPadding(0,1,0,0);
					pCtrlFirstItem->SetPadding(rcPadding);
				}
			}
		}
	}
}

BOOL CSettingPersonal::FindInFavoriteList(CString strGroupName)
{
	BOOL bFindRet = FALSE;
	for (int i = 0; i < m_pFavoriteRootNode->num_children(); ++i)
	{
		Node* childNode = m_pFavoriteRootNode->child(i);
		if (strGroupName == childNode->data().text_)
		{
			bFindRet = TRUE;
			break;
		}
		
	}
	return bFindRet;
}

const CString CSettingPersonal::GetNewGroupName(const CString strDefaultName)
{
	CString strGroupName;	
	int idx = 0;
	strGroupName = strDefaultName;
	while(1)
	{
		if(!FindInFavoriteList(strGroupName))
		{
			break;
		}		
		strGroupName.Format(_T("%s (%d)"), strDefaultName, ++idx);		//货 弊缝(0)...				
	}

	return strGroupName;		
}

void CSettingPersonal::SaveFavoriteFile()
{
	GT_MY_USER_DATA myUserData;
	g_pMessengerActor->GetMyUserInfo(myUserData);
	CString strDefaultName = myUserData.sMyPage.strUserName;
	CTime time(CTime::GetCurrentTime());
	GetLocaleTime(&time);
	strDefaultName += time.FormatGmt(_T("%Y%m%d"));
	std::vector<const IGTBuddyGroup* > vecGroup;
	FavoriteController* pFavoriteCtrl = GET_MESSENGER_DLG()->GetMainFrame()->GetFavoriteController();
	for (int i = 0; i < m_pExportRootNode->num_children(); ++i)
	{
		Node* childNode = m_pExportRootNode->child(i);
		const IGTBuddyGroup *pGroup = g_pMessengerActor->GetBuddyGroupInfo(childNode->data().text_.GetData());
		if (pGroup != NULL)
		{
			vecGroup.push_back(pGroup);
		}
	}
	m_pSettingFrame->SetEnableFlash(FALSE);
	pFavoriteCtrl->ExportGroup(vecGroup, strDefaultName, m_pManager->GetPaintWindow());
	m_pSettingFrame->SetEnableFlash(TRUE);
}

void CSettingPersonal::ImportFavoriteFile()
{
	m_pSettingFrame->SetEnableFlash(FALSE);
	FavoriteController* pFavoriteCtrl = GET_MESSENGER_DLG()->GetMainFrame()->GetFavoriteController();
	vector<IMPORT_GROUP>* pVecImportGroup = pFavoriteCtrl->GetGroupInfoFromFile(m_pManager->GetPaintWindow());
	if (pVecImportGroup != NULL)
	{
		vector<IMPORT_GROUP>::iterator it = pVecImportGroup->begin();
		for (; it!=pVecImportGroup->end(); ++it)
		{
			GroupItemInfo item;
			item.nGroupID = m_vecImportGroup.size();
			item.strGroupName = (*it).strGroupName.c_str();
			AddImportNode(item, NULL);
			m_vecImportGroup.push_back(*it);
		}
	}
	m_pSettingFrame->SetEnableFlash(TRUE);
}

void CSettingPersonal::EnableFavoriteList(bool bEnableNewFavorite, bool bEnableOldFavorite)
{
	if (m_nGroupType == PERSONAL_GROUP_EXPORT)
	{
		for (int i = 0; i < m_pFavoriteRootNode->num_children(); ++i)
		{
			CControlUI* pControl = m_pListFavorite->GetItemAt(i);
			//Node* node = NULL;
			if( pControl != NULL) 
			{
				Node* node = (Node*)pControl->GetTag();
				if(node != NULL)
				{
					if (g_pMessengerActor->GetBuddyGroupInfo(node->data().text_) == NULL)
					{
						CLabelUI* pLabGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pControl, kSettingPersonalGroupName));
						pLabGroupName->SetEnabled(false);
						pControl->SetEnabled(false);
					}
					else
					{
						CLabelUI* pLabGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pControl, kSettingPersonalGroupName));
						pLabGroupName->SetEnabled(true);
						pControl->SetEnabled(true);
					}
				}
			}
		}
		
	}
	else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
	{
		for (int i = 0; i < m_pFavoriteRootNode->num_children(); ++i)
		{
			CControlUI* pControl = m_pListFavorite->GetItemAt(i);
			//Node* node = NULL;
			if( pControl != NULL) 
			{
				Node* node = (Node*)pControl->GetTag();
				if(node != NULL)
				{
					if (g_pMessengerActor->GetBuddyGroupInfo(node->data().text_) == NULL)
					{
						CLabelUI* pLabGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pControl, kSettingPersonalGroupName));
						pLabGroupName->SetEnabled(true);
						pControl->SetEnabled(true);
					}
					else
					{
						CLabelUI* pLabGroupName = static_cast<CLabelUI*>(m_pManager->FindSubControlByName(pControl, kSettingPersonalGroupName));
						pLabGroupName->SetEnabled(false);
						pControl->SetEnabled(false);
					}
				}
			}
		}
	}
}

void CSettingPersonal::SetGroupType(int nGroupType)
{
	m_nGroupType = nGroupType;
	if (m_nGroupType == PERSONAL_GROUP_EXPORT)
	{
		CButtonUI* pBtnFile = static_cast<CButtonUI*>(m_pManager->FindControl(kSettingPersonalFile));
		if (pBtnFile != NULL)
		{
			pBtnFile->SetText(LoadStringFromResource(IDS_STRING_FILE_SAVE));
		}
		CLabelUI* pLabExport = static_cast<CLabelUI*>(m_pManager->FindControl(kSettingPersonalLabExport));
		if (pLabExport != NULL)
		{
			pLabExport->SetText(LoadStringFromResource(IDS_STRING_EXPORT_LIST));
		}
		m_pListExport->SetVisible(true);
		m_pListImport->SetVisible(false);
		EnableFavoriteList(false, true);
	}
	else if (m_nGroupType == PERSONAL_GROUP_IMPORT)
	{
		CButtonUI* pBtnFile = static_cast<CButtonUI*>(m_pManager->FindControl(kSettingPersonalFile));
		if (pBtnFile != NULL)
		{
			pBtnFile->SetText(LoadStringFromResource(IDS_STRING_FILE_OPEN));
		}
		CLabelUI* pLabExport = static_cast<CLabelUI*>(m_pManager->FindControl(kSettingPersonalLabExport));
		if (pLabExport != NULL)
		{
			pLabExport->SetText(LoadStringFromResource(IDS_STRING_IMPORT_LIST));
		}
		m_pListExport->SetVisible(false);
		m_pListImport->SetVisible(true);
		EnableFavoriteList(true, false);
	}
}

void CSettingPersonal::ProcLabelEvent(CControlUI* pCtrl)
{
	if( pCtrl != NULL ) 
	{
		if (_tcsicmp(pCtrl->GetName(), kSettingPersonalTimeButton) == 0)
		{
			m_bTimeCheck = !m_bTimeCheck;
			ShowTimeCheckStatus();
			CCheckBoxUI* pTimeCheck = static_cast<CCheckBoxUI*>(m_pManager->FindControl(kSettingPersonalTimeCheck));
			if (pTimeCheck != NULL)
			{
				pTimeCheck->SetCheck(m_bTimeCheck);
			}
			EnableTimeCombo();
		} 
	}
}

void CSettingPersonal::NotifyTextChange(TNotifyUI& msg)
{
	CString strText = msg.pSender->GetText();
	SetNameLength(strText.GetLength());
}

void CSettingPersonal::NotifyItemSelect(TNotifyUI& msg)
{
	CComboUI* pTimeCombo = static_cast<CComboUI*>(m_pManager->FindControl(kSettingPersonalTimeCombo));
	if (pTimeCombo != NULL)
	{
		int nSelect = pTimeCombo->GetCurSel();
		if (nSelect >= 0 && nSelect < MAX_AWAY_COUNT)
		{
			m_nAwayTimeSpin = c_nAwayTime[nSelect];
			GET_CONFIG()->SetAutoAwayTime(c_nAwayTime[nSelect]);
		}
		
	}
}

void CSettingPersonal::SetNameLength(int nLen)
{
	CLabelUI* pUserName = static_cast<CLabelUI*>(m_pManager->FindControl(_T("personal_name_length")));
	if (pUserName)
	{
		CString strText = _T("");
		strText.Format(_T("%d/20"), nLen);
		pUserName->SetText(strText);
	}
}

void CSettingPersonal::ShowTimeCheckStatus()
{
	CCheckBoxUI* pTimeCheck = static_cast<CCheckBoxUI*>(m_pManager->FindControl(kSettingPersonalTimeCheck));
	CTextUI* pTimeText = static_cast<CTextUI*>(m_pManager->FindControl(kSettingPersonalTimeButton));
	
	if (pTimeCheck != NULL && pTimeText != NULL)
	{
		/*
		CString strBuf = _T("");
		if (m_bTimeCheck)
		{
			strBuf.Format(_T("<f 1>%s</f><x 10><i btn_chat_talk_checkboc_on.png>"), LoadStringFromResource(IDS_STRING427));
		}
		else
		{
			strBuf.Format(_T("<f 1>%s</f><x 10><i btn_chat_talk_checkboc_off.png>"), LoadStringFromResource(IDS_STRING427));
		}
		pTimeCheck->SetShowHtml(true);
		*/
		pTimeText->OnEvent += MakeDelegate(OnTextEvent);
		pTimeText->SetTag(UINT_PTR(this));
		pTimeText->SetText(LoadStringFromResource(IDS_STRING427));
	}
}

void CSettingPersonal::EnableTimeCombo()
{
	CComboUI* pTimeCombo = static_cast<CComboUI*>(m_pManager->FindControl(kSettingPersonalTimeCombo));
	if (pTimeCombo != NULL)
	{
		pTimeCombo->SetEnabled(m_bTimeCheck);
	}
}

bool CSettingPersonal::DoApply()
{
	CEditUI* pEditUserName = static_cast<CEditUI*>(m_pManager->FindControl(_T("personal_edit_name")));
	CString strNickName = pEditUserName->GetText();
	if (strNickName.Find(_T('|')) >= 0)
	{
		//NMessageBox(CWnd::FromHandle(), LoadStringFromResource(IDS_STRING_MAIN_INVALIDNICKNAME));
		return false;
	}
	GET_CONFIG()->SetChangeNickName(strNickName);

	int nValue = m_bTimeCheck ? 1 : 0;
	GET_CONFIG()->SetAutoAwayState(nValue);
	if((GET_CONFIG()->GetAutoAwayState()) && (m_nAwayTimeSpin>0)) 
	{
		GET_MESSENGER_DLG()->m_IdleMonitor->ResumeMonitorInput();
	}
	else
	{
		GET_MESSENGER_DLG()->m_IdleMonitor->PauseMonitorInput();
	}

	GET_CONFIG()->SetChangeNickName(strNickName);
	FavoriteController* pFavoriteCtrl = GET_MESSENGER_DLG()->GetMainFrame()->GetFavoriteController();
	if (pFavoriteCtrl != NULL)
	{
		vector<IMPORT_GROUP> vecImportGroup;
		int nAddGroupBuddyCount = 0;
		for (size_t i=0; i<m_vecAddToFavorite.size(); ++i)
		{
			int nAddGroupIndex = m_vecAddToFavorite[i].nGroupID;
			if (nAddGroupIndex < m_vecImportGroup.size())
			{
				m_vecImportGroup[nAddGroupIndex].strGroupName = m_vecAddToFavorite[i].strGroupName;
				nAddGroupBuddyCount += m_vecImportGroup[nAddGroupIndex].vecBuddyID.size();
				vecImportGroup.push_back(m_vecImportGroup[nAddGroupIndex]);
			}
		}
		if (!vecImportGroup.empty())
		{
			int nNowBuddyCount = g_pMessengerActor->GetBuddyCount();
			if ((nNowBuddyCount + nAddGroupBuddyCount) > 300)
			{
				m_pSettingFrame->SetEnableFlash(FALSE);
				NMessageBox(CWnd::FromHandle(m_pManager->GetPaintWindow()), LoadStringFromResource(IDS_STRING632));
				m_pSettingFrame->SetEnableFlash(TRUE);
				return false;
			}
			g_pMessengerActor->RequestImportBuddy(vecImportGroup);
		}
	}

	return true;
}



